@@ -332,7 +332,11 @@ M.util.init_maximised_embed = function(Y, id) {
             if (val == 'auto') {
                 val = el.getComputedStyle(prop);
             }
-            return parseInt(val);
+            val = parseInt(val);
+            if (isNaN(val)) {
+                return 0;
+            }
+            return val;
         } else {
             return 0;
         }
@@ -351,7 +355,7 @@ M.util.init_maximised_embed = function(Y, id) {
 
         var headerheight = get_htmlelement_size('page-header', 'height');
         var footerheight = get_htmlelement_size('page-footer', 'height');
-        var newheight = parseInt(Y.one('body').get('winHeight')) - footerheight - headerheight - 100;
+        var newheight = parseInt(Y.one('body').get('docHeight')) - footerheight - headerheight - 100;
         if (newheight < 400) {
             newheight = 400;
         }
